﻿using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WCS.Core;
using WCS.Entity;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Protocol.BCR;
using WCS.WorkEngineering.Protocol.SRM;
using WCS.WorkEngineering.Protocol.Station;
using WCS.WorkEngineering.WebApi.Controllers;
using WCS.WorkEngineering.Worlds;
using static System.Runtime.InteropServices.JavaScript.JSType;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering.Systems
{
    /// <summary>
    ///  库前入库
    /// </summary>
    [BelongTo(typeof(MainWorld))]
    [Description("库前入库")]
    public class 库前入库 : DeviceSystem<Device<IStation520, IStation521, IStation523, IStation91, IBCR81>>
    {
        protected override bool ParallelDo => true;
        protected override bool SaveLogsToFile => true;
        private List<Station> devs = new List<Station>();
        private List<SRM> srms = new List<SRM>();
        public 库前入库()   
        {
            devs = Device.All.Where(v => v.HasFlag(DeviceFlags.入库)).Select(v => new Station(v, this.World)).ToList();
            srms = Device.All.Where(v => v.HasProtocol(typeof(ISRM520))).Select(v => new SRM(v, this.World)).ToList();
        }
        public override void Do(Device<IStation520, IStation521, IStation523, IStation91, IBCR81> obj)
        {
            obj.入库站点是否被禁止();
            obj.库前入库站点是否满足执行条件();
            string bcr = "";
            List<SRM> inlist = new List<SRM>();
            //叠盘机托盘组入库
            if (obj.Entity.Code == "8262")
            {
                if (obj.Data2.Request == 1)
                {
                    bcr = "TPC" + DateTime.Now.ToString("yyMMddHHmmss"); 
                }               
            }
            else
            {
                //开始读取条码
                bcr = obj.Data5.GetBCRCode();
            }
            if (bcr == null) throw new KnownException("未读取到条码", LogLevelEnum.High);
            WCS_TaskInfo task = null;
            SqlSugarHelper.Do(_db =>
            {
                var db = _db.Default;
                //找到对应任务
                var taskInfo = db.Queryable<WCS_TaskInfo>().First(x => x.BarCode == bcr && x.Type == TaskType.EnterDepot && x.AddrFrom == "8270");
                //未找到对应任务
                if (taskInfo == null)
                {
                    //向WMS申请入库任务
                    var res = WmsApi.I_WCS_GetInTask(bcr, obj.Entity.Code);
                    if (res.ResCode != WebApi.Models.WMS.Response.ResponseStatusCodeEnum.Sucess)
                    {
                        if (res == null) throw new Exception($"向WMS申请入库任务失败" + res.ResMsg);
                    }
                    return;
                }
                //任务生成
                if (taskInfo.Status == Entity.TaskStatus.WaitingToExecute)
                {
                    //开始获取堆垛机与可用站台信息
                    var res = WmsApi.GetTunnelPriorityList(taskInfo.ID);
                    if (res == null) throw new Exception($"未返回巷道");

                    var tunnelNo = res.ResData.Split(",").Select(v => "SRM" + v).ToList();
                    SRM srm = null;
                    List<Station> stations = new List<Station>();
                    //成品库库前堆垛机取货点
                    stations = devs.Where(v => v.Entity.Code is "8250" or "8249" or "8246" or "8244" or "8243").ToList();
                    //筛选出可用站台c
                    stations = stations.Where(v => v.Data.VoucherNo == v.Data2.VoucherNo && v.Data3.Status.HasFlag(StationStatus.Auto))
                                       .Where(v => !v.Data3.Status.HasFlag(StationStatus.PH_Status) && !v.Data3.Status.HasFlag(StationStatus.OT_Status))
                                       .Where(v => v.Data2.TaskNumber == 0)
                                       .Where(v => !v.Data3.Status.HasFlag(StationStatus.Run)).ToList(); // 筛选出可用站台
                    if (stations == null)
                    {
                        throw new KnownException($"{obj.Entity.Code}无可用放货站台", LogLevelEnum.High);
                    }
                    var srms = stations.Select(v => v.Entity.Targets).SelectMany(v => v).Where(v => v.IsTunnel()) //下一个地址是巷道的
                                  .Select(v => v.Sources).SelectMany(v => v).Where(v => v.HasProtocol(typeof(ISRM520))) //筛选出堆垛机
                                  .Select(v => new SRM(v, this.World)).ToList();
                    foreach (var device in srms)
                    {
                        try
                        {
                            var canusem = device.Data2.AutoStatus;
                                 
                            inlist.Add(device);
                        }
                        catch (Exception)
                        {
                            continue;
                        }
                    }
                    //可用堆垛机
                    srm = inlist.Where(v => v != null && !v.Data2.Status.HasFlag(SrmStatus.Alarm) && v.Data2.AutoStatus == SrmAutoStatus.Automatic) //筛选出可用堆垛机
                                  .Where(v => tunnelNo.Contains(v.Entity.Code)) //筛选出巷道优先级最高的堆垛机
                                  .MinBy(v => tunnelNo.IndexOf(v.Entity.Code)); //按照巷道优先级排序
                    if (srm == null)
                    {
                        throw new Exception($"任务{taskInfo.ID}无可用堆垛机");
                    }
                    inlist = null;
                    //筛选出堆垛机的取货站台
                    var nextPos = stations.FirstOrDefault(v => v.Entity.Targets.Where(t => t.IsTunnel()).Select(t => t.Sources).SelectMany(v => v) //获取所有巷道
                                                                               .Where(t => t.HasProtocol(typeof(ISRM520)))//筛选出堆垛机
                                                                               .SelectMany(v => v.Sources).Any(t => t.Code == srm.Entity.Parent.Code));
                    if (nextPos == null)
                    {
                        throw new KnownException($"{obj.Entity.Code}没有可用的放货站台", LogLevelEnum.High);
                    }

                    var dev = devs.Find(v => v.Entity.Code == nextPos.Entity.Code);
                    var nextCode = nextPos.Entity.Code;
                    if (dev == null) throw new Exception($"找不到放货点");                    

                    //堆垛机的可用取货站台
                    taskInfo.SrmStation = nextPos.Entity.Code;
                    taskInfo.AddrNext = nextPos.Entity.Targets.First().Code;
                    taskInfo.Device = srm.Entity.Code;
                    taskInfo.Tunnel = taskInfo.Device.GetLastDigit().ToString();
                    taskInfo.Status = Entity.TaskStatus.ConveyorExecution;
                    taskInfo.EditWho = "WCS";
                    taskInfo.EditTime = DateTime.Now;
                    db.Updateable(taskInfo).ExecuteCommand();
                    taskInfo.AddWCS_TASK_DTL(_db, obj.Entity.Code, taskInfo.AddrNext, $"完成巷道分配，分配至{taskInfo.Device}");
                    //taskInfo.updateRedisHash();
                    task = taskInfo;
                    if (task == null) throw new KnownException($"[{obj.Entity.Code}]等待入库任务为空", LogLevelEnum.Mid);
                }
            });
            if (task == null) return;
            obj.Data.TaskNumber = task.ID;
            obj.Data.GoodsStart = obj.Entity.Code.ToShort();
            obj.Data.GoodsEnd = task.SrmStation.ToShort();
            obj.Data.VoucherNo++;

        }
        public override bool Select(Device dev)
        {
            return dev.Code == "8270";
        }
    }
}
