﻿using ServiceCenter;
using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.Redis;
using ServiceCenter.WebApi;
using System.Threading.Tasks;
using WCS.WorkEngineering.WebApi.Models.AGV.Response;
using WCS.WorkEngineering.WebApi.Models.WCS.Response;
using WCS.WorkEngineering.WebApi.Models.WMS.Request;
using WCS.WorkEngineering.WebApi.Models.WMS.Response;

namespace WCS.WorkEngineering.WebApi.Controllers
{
    /// <summary>
    /// AGV相关接口控制器
    /// </summary>

    public static class WmsApi
    {
        private static string _WMSUrl = null!;
        private static string _wareHouseId = null!;
        private static string wmsUrl = "http://10.30.44.4:8070";
        /// <summary>
        ///  WMS URL
        /// </summary>
        public static string WMSUrl
        {
            get
            {
                _WMSUrl ??= RedisHub.Default.Check("WMSUrl");
                if (string.IsNullOrEmpty(_WMSUrl))
                {
                    throw new KnownException($"请在Redis配置WMSUrl", LogLevelEnum.High);
                }
                return _WMSUrl;
            }
        }

        /// <summary>
        /// 仓库编号
        /// </summary>
        public static string wareHouseId = ServiceHub.WarehouseName;

        /// <summary>
        ///  上传重量等信息
        /// </summary>
        /// <param name="taskCode">任务号</param>
        /// <param name="weight">rfid</param>
        /// <returns></returns>
        public static SRes WcsUploadInfo(int taskCode, decimal weight) => WcsUploadInfo(taskCode, weight, "");

        /// <summary>
        ///  上传重量等信息
        /// </summary>
        /// <param name="taskCode">任务号</param>
        /// <param name="RFID">重量</param>
        /// <returns></returns>
        public static SRes WcsUploadInfo(int taskCode, string RFID) => WcsUploadInfo(taskCode, 0, RFID);

        /// <summary>
        ///   上传重量等信息
        /// </summary>
        /// <param name="taskCode">任务号</param>
        /// <param name="weight">重量</param>
        /// <param name="RFID">rfid</param>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static SRes WcsUploadInfo(int taskCode, decimal weight, string RFID)
        {
            var res = APICaller.CallApi2<SRes>(wmsUrl + "/api/Cp/WcsUploadInfo", new WcsUploadInfoRequest
            {
                TaskCode = taskCode,
                Weight = weight,
                RFID = RFID
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  获取巷道
        /// </summary>
        /// <param name="wcsTaskNum">WMS任务ID</param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        public static SRes<string> GetTunnelPriorityList(int wcsTaskNum)
        {
            var res = APICaller.CallApi2<SRes<string>>(wmsUrl + "/api/Cp/GetTunnelPriorityList", new GetTunnelPriorityListRequest
            {
                TaskNum = wcsTaskNum,
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  分配货位
        /// </summary>
        /// <param name="wcsTaskNum">WMS任务ID</param>
        /// <param name="tunnel">货位</param>
        /// <param name="device">设备号</param>
        /// <param name="size"></param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        public static SRes<ApplyStockInLocResponse> GetLocalIn(int wcsTaskNum, string tunnel, string device)
        {
            var res = APICaller.CallApi2<SRes<ApplyStockInLocResponse>>(wmsUrl + "/api/Cp/ApplyStockInLoc", new ApplyStockInLocRequest
            {
                TaskNum = wcsTaskNum,
                TunnelNum = tunnel.GetLastDigit(),
                PickUpEquipmentNo = device.ToString(),
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  堆垛机出库任务执行完成
        /// </summary>
        /// <param name="taskNum"></param>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static SRes<int> SrmPickOutCompleted(int taskNum)
        {
            var res = APICaller.CallApi2<SRes<int>>(wmsUrl + "/api/Cp/SrmPickOutCompleted", new SrmPickOutCompletedRequest
            {
                TaskNum = taskNum,
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  一楼空工字轮申请出库
        /// </summary>
        /// <param name="outEndPostion">出库位置</param>
        /// <returns></returns>
        public static SRes ApplyStockOutTask(string outEndPostion) => ApplyStockOutTask(outEndPostion, 2);

        /// <summary>
        ///  申请出库任务
        /// </summary>
        /// <param name="outEndPostion">出库位置</param>
        /// <param name="outType">出库类型</param>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static SRes ApplyStockOutTask(string outEndPostion, int outType)
        {
            var request = new ApplyStockOutTaskRequest
            {
                OutEndPostion = outEndPostion,
                OutType = outType,
                WarehouseCode = wareHouseId,
            };
            switch (outEndPostion)
            {
                case "1012":
                    request.Tunnel = "1";
                    break;

                case "1014":
                    request.Tunnel = "2";
                    break;

                case "1016":
                    request.Tunnel = "3";
                    break;
            }
            var res = APICaller.CallApi2<SRes>(wmsUrl + "/api/Hj/ApplyStockOutTask", request);
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  获取各巷道剩余空轮数量
        /// </summary>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static SRes<GetTunnelEmptyConResponse> GetTunnelEmptyConCount()
        {
            var res = APICaller.CallApi2<SRes<GetTunnelEmptyConResponse>>(wmsUrl + "/api/Hj/GetTunnelEmptyConCount", new GetTunnelEmptyConCountRequest { });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  完成任务
        /// </summary>
        /// <param name="taskNo"></param>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static SRes<int> CompleteTask(int taskNo)
        {
            var res = APICaller.CallApi2<SRes<int>>(wmsUrl + "/api/Cp/CompleteTask", new CompleteTaskRequest
            {
                TaskNum = taskNo,
                OperationType = Models.WMS.Request.CompleteTask.自动完成,
                WCSUpdateName = "WCS"
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  二深位获取移库任务
        /// </summary>
        /// <param name="taskNo">任务</param>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static SRes<AddWcsMoveTaskResponse> AddWcsMoveTask(int taskNo)
        {
            var res = APICaller.CallApi2<SRes<AddWcsMoveTaskResponse>>(wmsUrl + "/api/Cp/WcsMoveTask", new CompleteTaskRequest
            {
                TaskNum = taskNo,
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        /// 向WMS获取入库任务 一次单卷
        /// </summary>
        /// <param name="barcode">产品条码</param>
        /// <param name="devCode">设备条码</param>
        /// <param name="getTunnel"></param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        public static SRes I_WCS_GetInTask(string barcode, string devCode, bool getTunnel = false)
        {
            try
            {
                var res = APICaller.CallApi<SRes>(wmsUrl + "/api/Cp/WCS_InTaskInfo", new GetInTaskInfoRequest()
                {
                    Code = barcode,
                    Equip = devCode,
                    WareHouse = "cphouse"

                });
                if (res.ResCode != ResponseStatusCodeEnum.Sucess)
                {
                    throw new KnownException(res.ResMsg, LogLevelEnum.High);
                }
                return res;
            }
            catch (Exception ex)
            {
                throw new KnownException(ex.Message, LogLevelEnum.High)
;
            }
        }
        public static SRes I_WCS_GetOutTask(string barcode, string devCode, bool getTunnel = false)
        {
            try
            {
                var res = APICaller.CallApi<SRes>(wmsUrl + "/api/Cp/PdaTrayTaskOut", new GetInTaskInfoRequest()
                {
                    Code = barcode,
                    Equip = devCode,
                    WareHouse = "cphouse"

                });
                if (res.ResCode != ResponseStatusCodeEnum.Sucess)
                {
                    throw new KnownException(res.ResMsg, LogLevelEnum.High);
                }
                return res;
            }
            catch (Exception ex)
            {
                throw new KnownException(ex.Message, LogLevelEnum.High)
;
            }
        }
        /// <summary>
        ///  处理任务验证接口
        /// </summary>
        /// <param name="taskNo">需要处理的任务进行验证</param>
        /// <param name="state">目标状态</param>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static SRes HandleTaskVerify(List<int> taskNo, int state)
        {
            var res = APICaller.CallApi2<SRes>(wmsUrl + "/api/Cp/CancelTaskVerify", new CancelTaskVerifyRequest
            {
                TaskNo = taskNo,
                State = state
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  处理任务接口
        /// </summary>
        /// <param name="taskNo">处理</param>
        /// <param name="state">目标状态</param>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static SRes CarryTaskInfo(List<int> taskNo, int state)
        {
            var res = APICaller.CallApi2<SRes>(wmsUrl + "/api/Cp/CarryTaskInfo", new CancelTaskVerifyRequest
            {
                TaskNo = taskNo,
                State = state
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  WMS完成或取消任务验证
        /// </summary>
        /// <param name="sRes"></param>
        /// <param name="id"></param>
        /// <param name="type">99完成,106取消</param>
        /// <returns></returns>
        public static SRes? HandleTaskVerify(SRes<HandleTaskResponse> sRes, int id, int type)
        {
            try
            {
                var res = WmsApi.HandleTaskVerify(new List<int>() { id }, type);
                return res;
            }
            catch (Exception ex)
            {
                sRes.ResDataList.Add(new HandleTaskResponse()
                {
                    IsSuccess = false,
                    TaskNo = id,
                    Message = $"WMS错误:{ex.Message}",
                });
                return null;
            }
        }

        /// <summary>
        ///  WMS完成或取消任务执行
        /// </summary>
        /// <param name="sRes"></param>
        /// <param name="id"></param>
        /// <param name="type">99完成,106取消</param>
        /// <returns></returns>
        public static SRes? CarryTaskInfo(SRes<HandleTaskResponse> sRes, int id, int type)
        {
            try
            {
                var res = WmsApi.CarryTaskInfo(new List<int>() { id }, type);
                return res;
            }
            catch (Exception ex)
            {
                sRes.ResDataList.Add(new HandleTaskResponse()
                {
                    IsSuccess = false,
                    TaskNo = id,
                    Message = $"WMS错误:{ex.Message}",
                });
                return null;
            }
        }

        /// <summary>
        ///  工字轮/芯股进入主线扫码
        /// </summary>
        /// <param name="codeList">工字轮条码组</param>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static SRes? OneFloorWorkerBuildEmptyPalletsStock(OneFloorWorkerBuildEmptyPalletsStockRequest reqDto)
        {
            var res = APICaller.CallApi2<SRes>(WMSUrl + "/api/FJ/OneFloorWorkerBuildEmptyPalletsStock", new OneFloorWorkerBuildEmptyPalletsStockRequest
            {
                PalletCode = reqDto.PalletCode,
                PalletType = reqDto.PalletType,
                StartLoc = reqDto.StartLoc,
                WareCode = GetWareCode(reqDto.StartLoc),
                PalletNum = "1"
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  获取仓库号
        /// </summary>
        public static string GetWareCode(string add)
        {
            return add switch
            {
                "2532" => "1N",
                "2527" => "1N",
                "2528" => "1N",
                "2732" => "1S",
                "2727" => "1S",
                "2728" => "1S",
                _ => "",
            };
        }

        /// <summary>
        ///  工字轮/芯股进入主线扫码
        /// </summary>
        /// <param name="reqDto"></param>
        /// <param name="equNo"></param>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static SRes EnteMainLine(List<string> reqDto, string equNo)
        {
            var res = APICaller.CallApi<SRes>(WMSUrl + "/api/FJ/EnteMainLine", new FJEnteMainLineRequest
            {
                IShapedWheelCodes = reqDto,
                equNo = equNo
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res;
        }

        /// <summary>
        ///  获取贴标数据
        /// </summary>
        /// <returns></returns>
        /// <exception cref="KnownException"></exception>
        public static ApplyLabelResponse GetLabelData(int id)
        {
            var res = APICaller.CallApi2<SRes<ApplyLabelResponse>>(wmsUrl + "/api/Cp/LabellingMachine", new
            {
                TaskNo = id
            });
            if (res.ResCode != ResponseStatusCodeEnum.Sucess)
            {
                throw new KnownException(res.ResMsg, LogLevelEnum.High);
            }
            return res.ResData;
        }
    }
}
